package com.tzld.longarticle.recommend.server.service.recommend;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.CostMonitor;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PushTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleUserGroupMapper;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.model.entity.crawler.PublishContentSortLog;
import com.tzld.longarticle.recommend.server.model.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.model.param.RecommendParam;
import com.tzld.longarticle.recommend.server.model.param.RecommendRequest;
import com.tzld.longarticle.recommend.server.model.vo.ArticleSortResponseData;
import com.tzld.longarticle.recommend.server.model.vo.ArticleSortResponseDataItem;
import com.tzld.longarticle.recommend.server.model.vo.RecommendResponse;
import com.tzld.longarticle.recommend.server.model.vo.RecommendWithUserGroupResponse;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishContentSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.common.constant.SceneConstants;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankService;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallParam;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallResult;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallService;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.tzld.longarticle.recommend.server.common.constant.SceneConstants.FWH_COLD_START;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecommendService {

    @Autowired
    private RecallService recallService;
    @Autowired
    private RankService rankService;
    @Autowired
    private PublishContentSortLogRepository publishContentSortLogRepository;
    @Autowired
    private PublishSortLogRepository publishSortLogRepository;
    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;
    @Autowired
    private ArticleUserGroupMapper articleUserGroupMapper;
    @Autowired
    private AigcBaseMapper aigcBaseMapper;
    @Autowired
    private ArticleRepository articleRepository;

    @ApolloJsonValue("${accountStrategyConfig:{}}")
    private Map<String, String> accountStrategyConfigMap;
    @ApolloJsonValue("${accountHisJumpStrategyConfig:[]]}")
    private List<String> accountHisJumpStrategyList;
    @Value("${spring.profiles.active}")
    private String env;


    public RecommendResponse recommend(RecommendRequest request) {
        long start = System.currentTimeMillis();
        RecommendParam param = genRecommendParam(request, SceneConstants.DEFAULT);
        log.info("genRecommendParam {}", JSONUtils.toJson(param));
        // 获取账号排序设置
        setStrategy(request, param);

        RecallResult recallResult = recallService.recall(convertToRecallParam(param));
        long t2 = System.currentTimeMillis();
        CostMonitor.logCost("Recommend", "Recall", t2 - start);
        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        long t3 = System.currentTimeMillis();
        CostMonitor.logCost("Recommend", "Rank", t3 - t2);
        saveSortLog(param, rankResult);

        RecommendResponse response = buildRecommendResponse(recallResult, rankResult, param.getPublishNum());
        long t4  = System.currentTimeMillis();
        log.info("recommendCost param:{} total cost:{} recall:{} rank:{} response: {}", JSONObject.toJSONString(request),
                t4 - start, t2 - start, t3 - t2, JSONObject.toJSONString(response));
        CostMonitor.logCost("Recommend", "Total", t4 - start);
        return response;
    }

    private void setStrategy(RecommendRequest request, RecommendParam param) {
        // 无限发表，设置为无限发表策略
        if (Objects.equals(request.getPushType(), PushTypeEnum.AUTO_PUBLISH.getVal())
                || Objects.equals(request.getPushType(), PushTypeEnum.ROBOPOST.getVal())) {
            param.setStrategy(RankStrategyEnum.INFINITE_STRATEGY.getStrategy());
            param.setType(ArticleTypeEnum.WUXIANLIU.getVal());
            return;
        }
        // 有账号策略，设置为账号策略
        String strategyConfig = accountStrategyConfigMap.get(request.getAccountName());
        if (StringUtils.hasText(strategyConfig) && !request.isParamStrategy()) {
            param.setStrategy(strategyConfig);
        }
        // 历史群发头条小于10条，且开启配置，则走历史表现随机策略
        int historyCount = articleRepository.countByGhIdAndTypeAndItemIndex(request.getGhId(),
                ArticleTypeEnum.QUNFA.getVal(), 1);
        if (historyCount < 10 && accountHisJumpStrategyList.contains(request.getGhId())) {
            param.setStrategy(RankStrategyEnum.HIS_JUMP_STRATEGY.getStrategy());
        }
        // 凌晨19点之后，设置为晚上策略
        if (DateUtils.getCurrentHour() >= 19 && !request.isParamStrategy()) {
            if (Objects.equals(request.getPushType(), PushTypeEnum.AUTO_GROUP_PUBLISH.getVal())) {
                param.setStrategy(RankStrategyEnum.LATE_STRATEGY.getStrategy());
            }
        }
    }

    public RecommendWithUserGroupResponse recommend4FwhColdStart(RecommendRequest request) {

        RecommendParam param = genRecommendParam(request, FWH_COLD_START);
        log.info("genRecommendParam {}", JSONUtils.toJson(param));
        // 获取账号排序设置
        String strategyConfig = accountStrategyConfigMap.get(request.getAccountName());
        if (StringUtils.hasText(strategyConfig)) {
            param.setStrategy(strategyConfig);
        }
        RecallResult recallResult = recallService.recall(convertToRecallParam(param));

        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));

        saveSortLog(param, rankResult);


        List<RecommendWithUserGroupResponse.RankData> rankList = new ArrayList<>();
        List<Content> contentList = rankResult.getContents();
        for (int i = 0, start = 0; i < param.getUserGroupIds().size() && start < contentList.size(); i++) {
            RecommendWithUserGroupResponse.RankData rankData = new RecommendWithUserGroupResponse.RankData();
            rankData.setUserGroupId(param.getUserGroupIds().get(i));
            int end = Math.min(contentList.size(), start + param.getPublishNum());
            rankData.setContentIds(CommonCollectionUtils.toList(contentList.subList(start, end), Content::getId));
            rankList.add(rankData);
            start += param.getPublishNum();
        }

        List<RecommendWithUserGroupResponse.FilterData> filterList = new ArrayList<>();
        for (RecallResult.RecallData data : recallResult.getData()) {
            if (CollectionUtils.isNotEmpty(data.getFilterContents())) {
                for (Content filterContent : data.getFilterContents()) {
                    RecommendWithUserGroupResponse.FilterData filterData = new RecommendWithUserGroupResponse.FilterData();
                    filterData.setContentId(filterContent.getId());
                    filterData.setFilterReason(filterContent.getFilterReason());
                    filterList.add(filterData);
                }
            }
        }


        return new RecommendWithUserGroupResponse(0, "success",
                new RecommendWithUserGroupResponse.RecommendWithUserGroupData(rankList, filterList));
    }

    private RecommendResponse buildRecommendResponse(RecallResult recallResult, RankResult rankResult, Integer publishNum) {
        List<Content> contentList = rankResult.getContents();
        List<ArticleSortResponseDataItem> rankList = new ArrayList<>();
        List<ArticleSortResponseDataItem> filterContentList = new ArrayList<>();
        for (RecallResult.RecallData data : recallResult.getData()) {
            if (CollectionUtils.isNotEmpty(data.getFilterContents())) {
                for (Content filterContent : data.getFilterContents()) {
                    ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                    BeanUtils.copyProperties(filterContent, item);
                    filterContentList.add(item);
                }
            }
        }
        ArticleSortResponseData data = new ArticleSortResponseData();
        if (CollectionUtils.isNotEmpty(contentList)) {
            for (Content content : contentList) {
                ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                BeanUtils.copyProperties(content, item);
                rankList.add(item);
            }
        }
        data.setRank_list(rankList.subList(0, Math.min(rankList.size(), publishNum)));
        data.setFilter_list(filterContentList);

        RecommendResponse response = new RecommendResponse();
        response.setCode(0);
        response.setMsg("success");
        response.setData(data);
        return response;
    }

    public RecommendParam genRecommendParam(RecommendRequest request, String abCode) {
        RecommendParam param = new RecommendParam();
        BeanUtils.copyProperties(request, param);
        param.setScene(abCode);

        switch (abCode) {
            case FWH_COLD_START:
                List<Integer> userGroupIds = articleUserGroupMapper.selectUserGroupIdByGzhId(request.getGhId());
                LocalDate today = LocalDate.now(); // 获取当前日期
                int dayOfYear = today.getDayOfYear();
                int tailNum = dayOfYear % 8;
                List<Integer> needPushUserGroupIds = new ArrayList<>();
                for (int userGroupId : userGroupIds) {
                    if (userGroupId % 8 == tailNum) {
                        needPushUserGroupIds.add(userGroupId);
                    }
                }
                param.setUserGroupIds(needPushUserGroupIds);
                break;
        }

        return param;
    }

    public RecallParam convertToRecallParam(RecommendParam param) {
        RecallParam recallParam = new RecallParam();
        BeanUtils.copyProperties(param, recallParam);
        recallParam.setStrategy(param.getStrategy());
        recallParam.setGhId(param.getGhId());
        return recallParam;
    }


    public RankParam convertToRankParam(RecommendParam param, RecallResult recallResult) {
        RankParam rankParam = new RankParam();
        List<Content> contentList = new ArrayList<>();
        List<Content> backup = new ArrayList<>();
        if (Objects.nonNull(recallResult) && CollectionUtils.isNotEmpty(recallResult.getData())) {
            for (RecallResult.RecallData item : recallResult.getData()) {
                if (item.isBackup()) {
                    backup.addAll(item.getContents());
                } else {
                    contentList.addAll(item.getContents());
                }
            }

        }
        rankParam.setContents(contentList);
        rankParam.setBackup(backup);
        rankParam.setStrategy(param.getStrategy());
        rankParam.setGhId(param.getGhId());
        rankParam.setAccountName(param.getAccountName());
        rankParam.setSize(param.getPublishNum());
        rankParam.setScene(param.getScene());
        rankParam.setUserGroupIds(param.getUserGroupIds());
        rankParam.setType(param.getType());

        return rankParam;
    }

    /**
     * 记录 账号、排序策略、结果。。。
     *
     * @param param
     * @param rankResult
     */
    private void saveSortLog(RecommendParam param, RankResult rankResult) {
        if (!"prod".equals(env) || param.isExcludeLog()) {
            return;
        }
        if (!ArticleTypeEnum.QUNFA.getVal().equals(param.getType())) {
            return;
        }
        switch (param.getScene()) {
            case FWH_COLD_START:
                break;
            default:
                PublishContentSortLog log = new PublishContentSortLog();
                log.setGhId(param.getGhId());
                log.setAccountName(param.getAccountName());
                log.setStrategy(param.getStrategy());
                List<String> publishContentIds = rankResult.getContents().stream().map(Content::getId).collect(Collectors.toList());
                log.setPublishContentId(JSONObject.toJSONString(publishContentIds));
                log.setCreateTimestamp(System.currentTimeMillis());
                publishContentSortLogRepository.save(log);
                break;

        }
        String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
        List<PublishSortLog> publishSortLogSaveList = new ArrayList<>();
        List<AccountAvgInfo> avgInfoList = accountAvgInfoRepository.getAllByGhIdEqualsAndStatusEquals(param.getGhId(), 1);
        List<PublishSortLog> hisSortLog = publishSortLogRepository.findByDateStrAndGhId(dateStr, param.getGhId());
        List<String> hisSortTitles = hisSortLog.stream().map(PublishSortLog::getTitle).collect(Collectors.toList());
        for (int i = 1; i < rankResult.getContents().size() + 1; i++) {
            Content content = rankResult.getContents().get(i - 1);
            if (CollectionUtils.isNotEmpty(hisSortLog)) {
                if (hisSortTitles.contains(content.getTitle())) {
                    continue;
                }
            }
            PublishSortLog sortLog = new PublishSortLog();
            sortLog.setDateStr(dateStr);
            sortLog.setGhId(param.getGhId());
            sortLog.setAccountName(param.getAccountName());
            sortLog.setCrawlerChannelContentId(content.getCrawlerChannelContentId());
            sortLog.setTitle(content.getTitle());
            sortLog.setIndex(i);
            sortLog.setIndexAvgCount(accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, param.getGhId(), i));
            if (CollectionUtils.isNotEmpty(content.getCategory())) {
                sortLog.setCategory(JSONObject.toJSONString(content.getCategory()));
            }
            sortLog.setStrategy(param.getStrategy());
            sortLog.setScore(String.valueOf(content.getScore()));
            sortLog.setScoreMap(JSONObject.toJSONString(content.getScoreMap()));
            sortLog.setCreateTimestamp(System.currentTimeMillis());
            publishSortLogSaveList.add(sortLog);
        }
        if (CollectionUtils.isNotEmpty(publishSortLogSaveList)) {
            publishSortLogRepository.saveAll(publishSortLogSaveList);
        }
    }

}
