package com.tzld.longarticle.recommend.server.util.feishu;

import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.util.MapBuilder;
import com.tzld.longarticle.recommend.server.util.feishu.model.Message;
import com.tzld.longarticle.recommend.server.util.feishu.model.MessageParams;
import com.tzld.longarticle.recommend.server.util.feishu.model.config.MessageConfig;
import com.tzld.longarticle.recommend.server.util.feishu.model.element.BaseElement;
import com.tzld.longarticle.recommend.server.util.feishu.model.element.DivElement;
import com.tzld.longarticle.recommend.server.util.feishu.model.element.Filed;
import com.tzld.longarticle.recommend.server.util.feishu.model.element.MarkDownText;
import com.tzld.longarticle.recommend.server.util.feishu.model.header.HeadTitle;
import com.tzld.longarticle.recommend.server.util.feishu.model.header.MessageHeader;
import com.tzld.longarticle.recommend.server.util.http.HttpClientUtils;
import com.tzld.longarticle.recommend.server.util.http.HttpResponseContent;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.apache.http.util.TextUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.*;

@Component
@Slf4j
public class FeishuMessageSender implements InitializingBean {

    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(FeishuMessageSender.class);

    private static final String webHookUrl = "https://open.feishu.cn/open-apis/bot/v2/hook/";

    private static final CloseableHttpClient client = HttpPoolFactory.defaultPool();

    private static String staticEnv;

    @Value("${spring.profiles.active:prod}")
    private String env;

    @Override
    public void afterPropertiesSet() throws Exception {
        FeishuMessageSender.staticEnv = env;
    }

    public void sendChat(String chatId, String title, Map<String, String> infos) {
        sendChat(chatId, null, title, infos);
    }

    public void sendChat(String chatId, String userOpenId, String title, Map<String, String> infos) {
        try {
            Pair<String, Integer> token = FeiShu.requestAccessToken();
            HashMap<String, String> header = new HashMap<>();
            header.put("Authorization", "Bearer " + token.getFirst());
            header.put("content-type", "application/json; charset=utf-8");

            MessageConfig messageConfig = new MessageConfig();
            MessageHeader messageHeader = new MessageHeader();
            messageHeader.title = new HeadTitle(title);
            List<BaseElement> elements = new ArrayList<>();
            DivElement textElement = new DivElement();
            for (Map.Entry<String, String> stringStringEntry : infos.entrySet()) {
                if (TextUtils.isBlank(stringStringEntry.getValue())) {
                    textElement.fields.add(new Filed(new MarkDownText(String.format("%s", stringStringEntry.getKey()))));
                } else {
                    textElement.fields.add(new Filed(new MarkDownText(String.format("**%s**：%s", stringStringEntry.getKey(), stringStringEntry.getValue()))));
                }
            }
            elements.add(textElement);
            MessageParams messageParams = new MessageParams(chatId, userOpenId, new Message(messageConfig, messageHeader, elements));
            HttpResponseContent hrc = HttpClientUtils.postRequestBody(FeiShu.URL_FEISHU_SEND_MESSAGE, messageParams, header);
        } catch (Exception e) {
            LOGGER.error("FeishuMessageSender", e);
        }
    }

    public static void sendWebHookMessage(String robotId, String msg) {
        // 使用自定义群机器人webhook方式，支持外部群
        if (!"prod".equals(staticEnv)) {
            return;
        }
        String webhookUrl = webHookUrl + robotId;
        String hrc = request(msg, webhookUrl);
        if (hrc == null) {
            // 重试一次
            hrc = request(msg, webhookUrl);
        }
        if (hrc == null) {
            log.error("sendWebHookMessage,hrc is null");
        }
    }

    public static String request(String msg, String webhookUrl) {
        long start = System.currentTimeMillis();
        JSONObject content = new JSONObject();
        content.put("text", msg);
        JSONObject bodyParam = new JSONObject();
        bodyParam.put("msg_type", "text");
        bodyParam.put("content", content);
        try {
            HttpPost httpPost = new HttpPost(webhookUrl);
            StringEntity stringEntity = new StringEntity(bodyParam.toJSONString(), StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    return EntityUtils.toString(responseEntity, "UTF-8");
                }
            }
        } catch (Exception e) {
            log.error("request error", e);
        }
        log.info("score耗时：{}", System.currentTimeMillis() - start);
        return null;
    }


    public static void main(String[] args) {
        FeishuMessageSender sender = new FeishuMessageSender();
        sender.sendChat("oc_b20f013b336e466fa572bc04aab13aba", "4090计算评分失败Test",
                MapBuilder.<String, String>builder(new LinkedHashMap<>())
                        .put("url", "http://61.48.133.26:6060/score_list")
                        .put("账号名称", "xxxxxx")
                        .build());

    }
}
